package cn.schedule.util;

import cn.common.exception.TaskException;
import cn.constant.ScheduleConst;
import cn.pojo.entity.TSysJob;
import cn.schedule.core.QuartzDisallowConcurrentExecution;
import cn.schedule.core.QuartzJobExecution;

import java.math.BigInteger;
import org.quartz.*;

/**
 * 定时任务工具类
 */
public class ScheduleUtil {
    /**
     * 得到quartz任务类
     *
     * @param sysJob 执行计划
     * @return 具体执行任务类
     */
    private static Class<? extends Job> getQuartzJobClass(TSysJob sysJob) 
    {
        Boolean isConcurrent = "0".equals(sysJob.getConcurrent());
        return isConcurrent ? QuartzJobExecution.class : QuartzDisallowConcurrentExecution.class;
    }
    /**
     * 创建定时任务
     */
    public static void createScheduleJob(Scheduler scheduler, TSysJob job) throws SchedulerException, TaskException 
    {
        Class<? extends Job> jobClass = getQuartzJobClass(job);
        // 构建job信息
        BigInteger jobId = job.getJobId();
        String jobGroup = job.getJobGroup();
        JobDetail jobDetail = JobBuilder.newJob(jobClass).withIdentity(getJobKey(jobId, jobGroup)).build();

        // 表达式调度构建器
        CronScheduleBuilder cronScheduleBuilder = CronScheduleBuilder.cronSchedule(job.getCronExpression());
        cronScheduleBuilder = handleCronScheduleMisfirePolicy(job, cronScheduleBuilder);

        // 按新的cronExpression表达式构建一个新的trigger
        CronTrigger trigger = TriggerBuilder.newTrigger()
        		 // 触发器名,触发器组 
        		.withIdentity(getTriggerKey(jobId, jobGroup))
        		//.startNow()
        		//触发器时间设定  
                .withSchedule(cronScheduleBuilder)
                .build();
        // 放入参数，运行时的方法可以获取
        jobDetail.getJobDataMap().put(ScheduleConst.TASK_PROPERTIES, job);

        // 判断是否存在
        if (scheduler.checkExists(getJobKey(jobId, jobGroup))) {
            // 防止创建时存在数据问题 先移除，然后在执行创建操作
            scheduler.deleteJob(getJobKey(jobId, jobGroup));
        }
        scheduler.scheduleJob(jobDetail, trigger);
        
        //暂停任务
        if (job.getStatus().equals(ScheduleConst.Status.PAUSE.getValue())) {
            scheduler.pauseJob(ScheduleUtil.getJobKey(jobId, jobGroup));
        }        
    }

    /**
     * 构建任务触发对象
     */
    public static TriggerKey getTriggerKey(BigInteger jobId, String jobGroup) {
        return TriggerKey.triggerKey(ScheduleConst.TASK_CLASS_NAME + jobId, jobGroup);
    }

    /**
     * 构建任务键对象
     */
    public static JobKey getJobKey(BigInteger jobId, String jobGroup) {
        return JobKey.jobKey(ScheduleConst.TASK_CLASS_NAME + jobId, jobGroup);
    }
    /**
     * 设置定时任务策略
     */
    public static CronScheduleBuilder handleCronScheduleMisfirePolicy(TSysJob job, CronScheduleBuilder cb)
                throws TaskException 
    {
        switch (job.getMisfirePolicy()) {
            case ScheduleConst.MISFIRE_DEFAULT:
                return cb;
            case ScheduleConst.MISFIRE_IGNORE_MISFIRES:
                return cb.withMisfireHandlingInstructionIgnoreMisfires();
            case ScheduleConst.MISFIRE_FIRE_AND_PROCEED:
                return cb.withMisfireHandlingInstructionFireAndProceed();
            case ScheduleConst.MISFIRE_DO_NOTHING:
                return cb.withMisfireHandlingInstructionDoNothing();
            default:
                throw new TaskException("The task misfire policy '" + job.getMisfirePolicy()
                        + "' cannot be used in cron schedule tasks", TaskException.Code.CONFIG_ERROR);
        }
    }
}
